//
// Cleversafe open-source code header - Version 1.2 - February 15, 2008
//
// Cleversafe Dispersed Storage(TM) is software for secure, private and
// reliable storage of the world's data using information dispersal.
//
// Copyright (C) 2005-2008 Cleversafe, Inc.
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either version 2
// of the License, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301,
// USA.
//
// Contact Information: Cleversafe, 224 North Desplaines Street, Suite 500 
// Chicago IL 60661
// email licensing@cleversafe.org
//
// END-OF-HEADER
//-----------------------
// Author: wleggette
//
// Date: Dec 21, 2007
//---------------------

package org.cleversafe.layer.access.cli;
import java.io.File;
import java.io.IOException;

import javax.management.InstanceNotFoundException;

import org.cleversafe.layer.access.GridAccessConnector;
import org.cleversafe.util.FileSystemUtils;

// TODO: Describe class or interface
public class CreateVault extends AbstractCLI
{	
   private static final int MAX_VAULT_DESCRIPTOR_BYTES = 1024 * 1024;
   
   private static final String COMMAND_NAME = "dsnet-create-vault";
   
   private static final String VAULT_NAME_OPTION = "vault_name";
   private static final String VAULT_DESCRIPTOR_OPTION = "vault_descriptor";
   
   
   ////////////////////////////////////////////////////////////////////////////////////////////////
   
   private final String vaultName;
   private final File vaultDescriptorFile;
   
   public CreateVault(String[] args) throws ExitStatusException
   {
      super(CreateVault.class, 
            COMMAND_NAME, 
            "create_vault_client.jsap", 
            "Creates a dsNet vault",
            args);
      this.vaultName = getJSAPResult().getString(VAULT_NAME_OPTION);
      this.vaultDescriptorFile = getJSAPResult().getFile(VAULT_DESCRIPTOR_OPTION); 
   }

   public CreateVault(String jmxHost, int jmxPort, String vaultName, File vaultDescriptorFile)
   {
      super(COMMAND_NAME, jmxHost, jmxPort);
      this.vaultName = vaultName;
      this.vaultDescriptorFile = vaultDescriptorFile;
   }

   public void run()
   {
      String descriptor;
      try
      {
    	 if(this.vaultDescriptorFile.length() > MAX_VAULT_DESCRIPTOR_BYTES)
    	 {
    		 throw new ExitStatusException("Vault descriptor exceeds size threshold", SECURITY_ERROR);
    	 }
         descriptor = FileSystemUtils.readFileAsString(this.vaultDescriptorFile);
      }
      catch (IOException e)
      {
         throw new ExitStatusException("Could not read vault descriptor file", IO_ERROR, e);
      }
      
      GridAccessConnector connector = this.getGridAccessConnector();
       
      try
      {
         connector.connect();
         
         String id = connector.getVaultStatusMBean().createVault(descriptor, this.vaultName);
         
         System.out.println("Successfully created a dsNet vault.");
         System.out.println("Vault name:       " + this.vaultName);
         System.out.println("Vault identifier: " + id);
      }
      catch (InstanceNotFoundException e)
      {
         throw new ExitStatusException("Error connecting to access server", IO_ERROR, e);
      }
      catch (Exception e)
      {
         throw new ExitStatusException("Error creating vault on access server", UNKNOWN_ERROR, e);
      }
      finally
      {
         try
         {
            if (connector != null)
            {
               connector.close();
            }
         }
         catch (IOException e) {}
      }
   }
   
   public static void main(String[] args)
   {
      try
      {
         CreateVault client = new CreateVault(args);
         client.finish();
      }
      catch (ExitStatusException e)
      {
         System.exit(e.getStatus());
      }
   }
   
   
   
}


