//
// Cleversafe open-source code header - Version 1.2 - February 15, 2008
//
// Cleversafe Dispersed Storage(TM) is software for secure, private and
// reliable storage of the world's data using information dispersal.
//
// Copyright (C) 2005-2008 Cleversafe, Inc.
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either version 2
// of the License, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301,
// USA.
//
// Contact Information: Cleversafe, 224 North Desplaines Street, Suite 500 
// Chicago IL 60661
// email licensing@cleversafe.org
//
// END-OF-HEADER
//-----------------------
// Author: Jason
//
// Date: May 15, 2007
//---------------------

package org.cleversafe.storage.ss.handlers.transaction;

import java.util.ArrayList;
import java.util.List;

import org.apache.log4j.Logger;
import org.cleversafe.layer.protocol.BeginTransactionRequest;
import org.cleversafe.layer.protocol.BeginTransactionResponse;
import org.cleversafe.layer.protocol.Request;
import org.cleversafe.layer.protocol.Response;
import org.cleversafe.layer.slicestore.SliceStore;
import org.cleversafe.layer.slicestore.SliceStoreTransaction;
import org.cleversafe.layer.slicestore.exceptions.SliceStoreIOException;
import org.cleversafe.layer.slicestore.exceptions.SliceStoreLayerException;
import org.cleversafe.layer.slicestore.exceptions.SliceStoreTransactionException;
import org.cleversafe.server.ApplicationType;
import org.cleversafe.server.ClientSession;
import org.cleversafe.server.RequestHandler;
import org.cleversafe.server.exceptions.IllegalSessionStateException;

/**
 * Handler for BeginTransactionRequest message
 * 
 * @author Jason Resch
 */
public class BeginTransactionHandler implements RequestHandler
{
   private static final Logger _logger = Logger.getLogger(BeginTransactionHandler.class);

   private final List<ApplicationType> _allowedApplications;

   /**
    * Constructs object and creates allowed applications access control list
    */
   public BeginTransactionHandler()
   {
      this._allowedApplications = new ArrayList<ApplicationType>();
      this._allowedApplications.add(ApplicationType.TYPE_AUTHENTICATED);
   }

   /**
    * Returns class for the type of message this handler services
    * 
    * @see RequestHandler.getRequestClass()
    */
   public Class<BeginTransactionRequest> getRequestClass()
   {
      return BeginTransactionRequest.class;
   }

   /**
    * Returns a list of application types that can access this handler
    * 
    * @see RequestHandler.getAllowedApplications()
    */
   public List<ApplicationType> getAllowedApplications()
   {
      return this._allowedApplications;
   }

   /**
    * Calls SliceStore.createTransaction() and then uses the created transaction to call
    * SliceStore.beginTransaction() on the SliceStore saved in the client session.
    * 
    * @see RequestHandler.service(Request request, ClientSession session)
    */
   public Response service(Request request, ClientSession session)
         throws IllegalSessionStateException, SliceStoreTransactionException, SliceStoreIOException
   {
      if (_logger.isTraceEnabled())
         _logger.trace("Servicing begin transaction request");

      // Check pre-conditions
      if (!session.containsKey(ClientSession.SLICE_STORE))
      {
         throw new IllegalSessionStateException(
               "A slice store has not been bound to this session.", request);
      }

      // Process request
      SliceStore sliceStore = (SliceStore) session.get(ClientSession.SLICE_STORE);

      BeginTransactionRequest beginTransactionRequest = (BeginTransactionRequest) request;

      // TODO: We may want to separate the create transaction and begin transaction
      // requests to conform more closely with how the SliceStore works
      long id = beginTransactionRequest.getTransactionID();

      // Perform Begin Transaction
      BeginTransactionResponse response = new BeginTransactionResponse();

      try
      {
         SliceStoreTransaction transaction = sliceStore.createTransaction(id);
         sliceStore.beginTransaction(transaction);
      }
      catch (SliceStoreLayerException ex)
      {
         _logger.error("Begin txid=" + id + "failed" + ex.getMessage());
         response.setException(ex);
      }

      if (_logger.isTraceEnabled())
         _logger.trace("Transaction begin with ID " + id);

      return response;
   }

}
